<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2005 Felix Nagel (info@felixnagel.com)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

require_once(PATH_tslib.'class.tslib_pibase.php');

// check for loaded t3jquery extension		
if (t3lib_extMgm::isLoaded('t3jquery')) {
	require_once(t3lib_extMgm::extPath('t3jquery').'class.tx_t3jquery.php');
}


/**
 * Plugin 'Sourcecode (beautyOfCode)' for the 'beautyofcode' extension.
 *
 * @author	Felix Nagel <info@felixnagel.com>
 * @package	TYPO3
 * @subpackage	tx_beautyofcode
 */
class tx_beautyofcode_pi1 extends tslib_pibase {
	var $prefixId      = 'tx_beautyofcode_pi1';		// Same as class name
	var $scriptRelPath = 'pi1/class.tx_beautyofcode_pi1.php';	// Path to this script relative to the extension dir.
	var $extKey        = 'beautyofcode';	// The extension key.
	var $pi_checkCHash = true;
	
	/**
	 * The main method of the PlugIn
	 *
	 * @param	string		$content: The PlugIn content
	 * @param	array		$conf: The PlugIn configuration
	 * @return	The content that is displayed on the website
	 */
	 
	function main($content,$conf) {
		$this->conf = $conf;
		$this->pi_setPiVarDefaults();
		$this->pi_loadLL();
		
		// check for static template
		$this->checkForStaticTempl();
		
		// parse XML data into php array
		$this->pi_initPIflexForm();
		
		// get configs (Ext Manager, Flexform)
		$this->flexFormValue = $this->getflexFormValue();
		$this->extManConf = $this->getExtManConf();
		// merge Extension Manager configs with possible set TS
		$this->mergeExtConfWithTS();	
		
		// set Header data
		$this->setHeaderData();		
		
		//create preview
		$this->makePreview();		
		
		// return code snippet as content
		return $this->cObj->stdWrap($this->getHTML(),$conf);
	}
	
	/**
	 * Function set header data (JS files and JS HTML)
	 */
	private function setHeaderData() {
		
		// empty header, js needs to be included once
		$GLOBALS['TSFE']->additionalHeaderData[$this->prefixId] = null;
		
		// please note only the jquery core js is included by t3jquery. 
		// All other components added manually cause of more flexibility
		if (T3JQUERY === true) {
			// add jQuery core by t3jquery extension
			tx_t3jquery::addJqJS();
		} else {
			// add jQuery core manually if defined
			if ($this->conf['addjQuery']>0) {
				$GLOBALS['TSFE']->additionalHeaderData[$this->prefixId] .= '<script src="'.$GLOBALS['TSFE']->tmpl->getFileName("EXT:beautyofcode/res/jquery-1.3.2.min.js").'" type="text/javascript"></script>';
			}
		}
		
		// add jquery.beautyOfCode.js
		$GLOBALS['TSFE']->additionalHeaderData[$this->prefixId] .='<script type="text/javascript" src="'.$this->extManConf['scriptUrl'].'"></script>';
			
		// get defaults
		$defaults = $this->getDefaults();
		
		// choose jQuery function selector
		$jQvar = ($this->conf['noConflict']) ? "jQuery" : "$";
		
		// additional selector?
		// $jQuerySelector = (strlen(trim($this->conf['jQuerySelector']))>0) ? trim($this->conf['jQuerySelector']) . ' ' : FALSE;
		
		$jsCode = "\n";
		// add noConflict jQuery code
		if ($this->conf['noConflict']) {
			$jsCode											.= $jQvar.'.noConflict();'."\n";
		}
		$jsCode 											.= $jQvar.'(document).ready(function(){'."\n";
		$jsCode 											.= "\t".$jQvar.'.beautyOfCode.init({'."\n";
		if (!empty($this->extManConf['baseUrl'])) $jsCode 	.= "\t\t".'baseUrl: "'.$this->extManConf['baseUrl'].'",'."\n";
		if (!empty($this->extManConf['scripts'])) $jsCode 	.= "\t\t".'scripts: "'.$this->extManConf['scripts'].'",'."\n";
		if (!empty($this->extManConf['styles'])) $jsCode 	.= "\t\t".'styles: "'.$this->extManConf['styles'].'",'."\n";
		if (strlen(trim($this->conf['theme']))> 0) $jsCode 	.= "\t\t".'theme: "'.trim($this->conf['theme']).'",'."\n";
		if (!empty($this->extManConf['defaults'])) $jsCode 	.= "\t\t".'defaults: {'.$defaults.'},'."\n";
		// if ($jQuerySelector) $jsCode 						.= "\t\t".'addSelector: "'.$jQuerySelector.'",'."\n";
		$jsCode 											.= "\t\t".'brushes: ["Plain"'.$this->getBrushes().']'."\n";
		$jsCode 											.= "\t".'});';
		$jsCode 											.= '});';
		
		// add js init
		$GLOBALS["TSFE"]->setJS($this->extKey, $jsCode);
	}
	
	/**
	 * Function to set HTML code
	 *
	 * @return	string  contains build html with sourcecode
	 */
	private function getHTML() {	
		// make html
		$content='';
		$lang = ($this->flexFormValue['lang']) ? $this->flexFormValue['lang'] : $lang = "plain";	
		$content .= "\n".'<pre class="code"><code class="'.$lang.$this->getCssConfig().'">'."\n";
		$content .= htmlentities($this->flexFormValue['code'])."\n";
		$content .= '</code></pre>'."\n";		
		
		// make label
		$labelConfig = $this->conf['label.'];
		if ($labelConfig['show'] && trim($this->flexFormValue['label']) != "") $content .= "\n".$this->cObj->stdWrap($this->flexFormValue['label'], array("wrap" => $labelConfig['wrap']))."\n";
		
		return $content;
	}
	
	/**
	 * Function to get Extension Manager settings
	 *
	 * @return	array contains extension manager settings
	 */
	private function getExtManConf() {
		$temp = array();
	
		// get values set in extension manager module
		$extManConf = isset($GLOBALS["TYPO3_CONF_VARS"]["EXT"]["extConf"]["beautyofcode"]) ? unserialize($GLOBALS["TYPO3_CONF_VARS"]["EXT"]["extConf"]["beautyofcode"]) : array();
		
		// add source file
		$temp['scriptUrl'] = (strlen(trim($extManConf['scriptUrl']))>0) ? $this->makeAbsolutePath(trim($extManConf['scriptUrl'])) : $this->makeAbsolutePath("EXT:beautyofcode/res/jquery.beautyOfCode.js");
		
		// if empty online repo is used
		$temp['baseUrl'] = (strlen(trim($extManConf['baseUrl']))>0) ? t3lib_div::getIndpEnv('TYPO3_SITE_URL').$this->makeAbsolutePath(trim($extManConf['baseUrl'])) : "";
		
		$temp['scripts'] = (strlen(trim($extManConf['scripts']))>0 && trim($extManConf['scripts']) != "scripts/") ? trim($extManConf['scripts']) : "";
		
		$temp['styles'] = (strlen(trim($extManConf['styles']))>0 && trim($extManConf['styles']) != "styles/") ? trim($extManConf['styles']) : "";
		
		return $temp;
	}
	
	/**
	 * Function to merge TS with Extension Manager configs
	 *
	 */
	private function mergeExtConfWithTS() {		
		if (strlen($this->conf['scriptUrl'])>0) $this->extManConf['scriptUrl'] = $this->makeAbsolutePath(trim($this->conf['scriptUrl']));
		
		if (strlen($this->conf['baseUrl'])>0) $this->extManConf['baseUrl'] = t3lib_div::getIndpEnv('TYPO3_SITE_URL').$this->makeAbsolutePath(trim($this->conf['baseUrl']));
		
		if (strlen(trim($this->conf['scripts']))>0 && trim($this->conf['scripts']) != "scripts/") $this->extManConf['scripts'] = trim($this->conf['scripts']);
		
		if (strlen(trim($this->conf['styles']))>0 && trim($this->conf['styles']) != "styles/") $this->extManConf['styles'] = trim($this->conf['styles']);
	}
	
	/**
	 * Function to get get flexform values
	 *
	 * @return	array contains flex form valuees
	 */
	private function getflexFormValue() {
		$temp = array();
		
		// get flexform values
		$piFlexForm = $this->cObj->data['pi_flexform'];
		$temp['label'] = $this->pi_getFFvalue($piFlexForm, 'cLabel', 'sDEF');
		$temp['lang'] = $this->pi_getFFvalue($piFlexForm, 'cLang', 'sDEF');
		$temp['code'] = $this->pi_getFFvalue($piFlexForm, 'cCode', 'sDEF');
		
		$temp['css'] = array();
		$temp['css']['highlight'] = $this->pi_getFFvalue($piFlexForm, 'cHighlight', 'sOPTIONS');
		$temp['css']['gutter'] = $this->pi_getFFvalue($piFlexForm, 'cGutter', 'sOPTIONS');
		$temp['css']['toolbar'] = $this->pi_getFFvalue($piFlexForm, 'cToolbar', 'sOPTIONS');
		$temp['css']['collapse'] = $this->pi_getFFvalue($piFlexForm, 'cCollapse', 'sOPTIONS');
		
		return $temp;
	}
	
	/**
	 * Function to make preview in BE FCE Element
	 *
	 */
	private function makePreview() {
		$preview = ($this->flexFormValue['label']) ? "[".$this->flexFormValue['lang']."] ".substr($this->flexFormValue['label'],0,110) : "[".$this->flexFormValue['lang']."] ".htmlentities(substr($this->flexFormValue['code'],0,120));
		$bodytext = $this->cObj->data['bodytext'];
		if($bodytext != $preview) {
			#copy the code to bodytext for preview in BE
			$res = $GLOBALS['TYPO3_DB']->exec_UPDATEquery('tt_content','uid='.$this->cObj->data['uid'],array('bodytext'=>$preview)); 
		}
	}
	
	/**
	 * Function to solve CSSconfiguration which overwrites TS configuration 
	 *
	 * @return	string  space seperated CSS classes
	 */
	private function getCssConfig() {
		// built brushes string
		$temp = '';
		foreach($this->flexFormValue['css'] AS $config => $configValue) { 
			if ($configValue != "" && $configValue != "auto") {
				if ($config == "highlight") {
					$temp2 = t3lib_div::intExplode(",", $configValue);
					$temp .= " boc-highlight[".implode(",", $temp2)."]";
				} else {
					if ($configValue) $temp .= " boc-".$config;
					else $temp .= " boc-no-".$config;
				}
			}
		}
		return $temp;
	}
	
	/**
	 * Function to solve brushes 
	 *
	 * @return	string  comma seperated list of the brushes with double quotes
	 */
	private function getBrushes() {
		// built brushes string
		$temp = '';
		if (strlen($this->conf['brushes'])>0) {
			$brushesArray = explode(',', $this->conf['brushes']);
			foreach($brushesArray AS $brush) { 
				$temp .= ',"'.trim($brush).'"';			
			}
		}
		return $temp;
	}
	
	/**
	 * Function to solve defaults 
	 *
	 * @return	string  comma seperated list list of the brushes with double quotes with no ending ,
	 */
	private function getDefaults() {
		$temp = '';
		if (is_array($this->conf['defaults.'])>0) {
			foreach($this->conf['defaults.'] AS $key => $value) { 
				$temp .= '"'.$key.'": '.$value.',';			 
			} 
			$temp = substr($defaults, 0, -1);
		}		
		return $temp;
	}
	
	/**
	 * Function to solve path with FILE: and EXT: 
	 *
	 * @param	string	path to directory
	 * @return	string
	 */
	private function makeAbsolutePath($dir) {
		if (t3lib_div::isFirstPartOfStr($dir, 'EXT:'))	{
			list($extKey,$script)=explode('/',substr($dir,4),2);
			if ($extKey && t3lib_extMgm::isLoaded($extKey))	{
				$extPath=t3lib_extMgm::extPath($extKey);
				return substr($extPath,strlen(PATH_site)).$script;
			}
		} elseif (t3lib_div::isFirstPartOfStr($dir, 'FILE:')) {
				return substr($dir,5);					
		} else {
			return $dir;
		}	
	}
	
	/**
	 * Function to check if static template is included
	 *
	 */
	private function checkForStaticTempl() {
		
		if (is_array($this->conf)
			&& !isset($this->conf["noConflict"])
			&& !isset($this->conf["addjQuery"])
			&& !isset($this->conf["label"])
			&& !isset($this->conf["wrap"])
			&& !isset($this->conf["theme"])
			&& !isset($this->conf["brushes"])
			&& !isset($this->conf["defaults"])
		) {
			t3lib_div::syslog('Please add static template as beautyOfCode will not work without! (page id = ' . $this->cObj->data['pid'] . ')', $this->extKey, 4);
			return $this->cObj->stdWrap('<div style="color: red; margin: 10px; padding: 10px; background: white; border: 3px solid red;"><strong>Beaty of Code Extension Error</strong><br /><p>Please add static TS to your main template!</p></div>');
		}
	}
}


if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/beautyofcode/pi1/class.tx_beautyofcode_pi1.php'])	{
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/beautyofcode/pi1/class.tx_beautyofcode_pi1.php']);
}

?>